@@ -1,10 +1,11 @@
 package Spreadsheet::HTML;
 use strict;
 use warnings FATAL => 'all';
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use Data::Dumper;
 use HTML::Entities;
+use Math::Matrix;
 
 sub new {
     my $class = shift;
@@ -12,28 +13,30 @@ sub new {
     return bless { %attrs }, $class;
 }
 
-sub table {
+sub generate {
     my ($self, @data);
 
     if (ref($_[0]) eq __PACKAGE__) {
         $self = shift;
-        @data = @{ $self->{data} };
+        @data = $self->get_data;
     } else {
         @data = get_data( @_ );
     }
 
-    my $header = '<tr>';
-    $header .= qq{<th>$_</th>} for @{ shift @data }; 
-    $header .= '</tr>';
+    return _make_table( @data );
+}
 
-    my $rows = '';
-    for (@data) {
-        $rows .= '<tr>';
-        $rows .= qq{<td>$_</td>} for @$_;
-        $rows .= '</tr>';
+sub transpose {
+    my ($self, @data);
+
+    if (ref($_[0]) eq __PACKAGE__) {
+        $self = shift;
+        @data = $self->get_data;
+    } else {
+        @data = get_data( @_ );
     }
 
-    return '<table>' . $header . $rows . '</table>';
+    return _make_table( @{ Math::Matrix::transpose(\@data) } );
 }
 
 sub get_data {
@@ -42,8 +45,8 @@ sub get_data {
     if (ref($_[0]) eq __PACKAGE__) {
         # called as method
         $self = shift;
+        return @{ $self->{data} } if $self->{__processed_data__};
         @data = @{ ref($self->{data}) ? $self->{data} : [[ $self->{data} ]] };
-        return @data if $self->{__processed_data__}++;
     } else {
         # called as function
         @data = @_ > 1 ? @_ : @{ ref($_[0]) ? $_[0] : [[ $_[0] ]] };
@@ -51,12 +54,37 @@ sub get_data {
 
     @data = [ @data ] unless ref( $data[0] ) eq 'ARRAY';
 
-    $self->{data} = _process_data( @data );
+    $self->{data} = _encode( @data );
+    $self->{data} = _mark_headers( $self->{data} );
+    $self->{__processed_data__} = 1;
     return @{ $self->{data} };
 }
 
+sub _make_table {
+    my $rows = '';
+    for my $row (@_) {
+        $rows .= '<tr>';
+        for my $cell (@$row) {
+            if (ref($cell)) {
+                $rows .= sprintf( '<th>%s</th>', $cell->[0] );
+            } else {
+                $rows .= sprintf( '<td>%s</td>', $cell );
+            }
+        }
+        $rows .= '</tr>';
+    }
 
-sub _process_data {
+    return "<table>$rows</table>";
+}
+
+sub _mark_headers {
+    my $data = shift;
+    $data->[0] = [ map [$_], @{ $data->[0] } ];
+    return $data;
+}
+
+
+sub _encode {
     my @data = @_;
 
     # padding is determined by first row (the header)
@@ -87,11 +115,21 @@ Spreadsheet::HTML - Tabular data to HTML tables.
 
 =head1 SYNOPSIS
 
-  use Spreadsheet::HTML;
+    use Spreadsheet::HTML;
 
-  my $table = Spreadsheet::HTML->new( data => 'AoA' );
+    my $data = [
+        [qw(header1 header2 header3)],
+        [qw(foo bar baz)],
+        [qw(one two three)],
+        [qw(col1 col2 col3)],
+    ];
 
-  print $table->generate;
+    my $table = Spreadsheet::HTML->new( data => $data );
+    print $table->generate;
+    print $table->transpose;
+
+    print Spreadsheet::HTML::generate( $data );
+    print Spreadsheet::HTML::transpose( $data );
 
 =head1 METHODS
 
@@ -99,9 +137,25 @@ Spreadsheet::HTML - Tabular data to HTML tables.
 
 =item new()
 
+Constructs object. Currently accepts one parameter: data.
+Data should be a two dimensional array and you should 
+expect the first row to be treated as the header (which
+means each cell will be wrapped with <th> tags instead 
+of <td> tags).
+
+=item generate()
+
+Returns a string that contains the rendered HTML table.
+
+=item transpose()
+
+Uses Math::Matrix to transpose the data and then returns
+a string containing the rendered HTML table.
+
 =item get_data()
 
-=item table()
+Should be an internal private method, but returns the 
+munged data before it is used to generate the HTML table.
 
 =back
 
